package
{
  import com.salesforce.AsyncResponder;
  import com.salesforce.Connection;
  import com.salesforce.Util;
  import com.salesforce.events.DebugEvent;
  import com.salesforce.objects.LoginRequest;
  import com.salesforce.objects.SObject;
  import com.salesforce.results.Fault;
  import com.salesforce.results.LoginResult;
  import com.salesforce.results.QueryResult;

  import flash.events.Event;
  import flash.events.EventDispatcher;

  import mx.collections.ArrayCollection;
  import mx.rpc.events.FaultEvent;
  import mx.rpc.events.ResultEvent;
  import mx.utils.ObjectUtil;
  import mx.utils.UIDUtil;

  public class Controller extends EventDispatcher
  {
    private var model:Model;
    private var conn:Connection;

    public function Controller()
    {
      model = Model.getInstance();
      conn = new Connection();
    }

    public function login(sessionId:String, serverUrl:String):void
    {
      conn.serverUrl = serverUrl;

      var lr:LoginRequest = new LoginRequest();
      lr.session_id = sessionId;
      lr.server_url = serverUrl;
      lr.callback = new AsyncResponder(loginResult, loginFault);
      conn.login(lr);
    }

    private function loginResult(result:LoginResult):void
    {
      if (!model.loggedIn)
      {
        model.statusMessage = "Logged In - Fetching Data";
        model.loggedIn = true;

        getAccidentReports();
      }
    }

    private function loginFault(fault:Fault):void
    {
      model.statusMessage = "Problem Logging In: " + fault.faultstring;
      model.loggedIn = false;
    }

    public function getAccidentReports():void
    {
      model.statusMessage = "Select an Accident Report";

      // get AccidentReports
      conn.query("Select Id, IncidentX__c, IncidentY__c, Date_of_Accident__c from AccidentReport__c", new AsyncResponder(getAccidentReportsResult, queryFault));
    }

    private function getAccidentReportsResult(qr:QueryResult):void
    {
      model.accidentReports = new ArrayCollection();

      // store AccidentReports
      for each (var o:Object in qr.records)
      {
        model.accidentReports.addItem(new AccidentReport(o));
      }
    }

    public function createAccidentReport():void
    {
      var accidentReportSObject:SObject = new SObject("AccidentReport__c");
      accidentReportSObject.IncidentX__c = 170;
      accidentReportSObject.IncidentY__c = 175;

      conn.create([accidentReportSObject], new AsyncResponder(createAccidentReportResult, createAccidentReportFault));
    }

    private function createAccidentReportResult(o:Object):void
    {
      getAccidentReports();
    }

    private function createAccidentReportFault(fault:Fault):void
    {
      model.statusMessage = "Problem Creating Accident Report In: " + fault.faultstring;
    }

    public function getAccidentReport(accidentReport:AccidentReport):void
    {
      model.selectedAccidentReport = accidentReport;

      // get AccidentReport's cars
      conn.query("Select Id, StartX__c, StartY__c, EndX__c, EndY__c from IncidentCar__c where AccidentReport__r.Id = '" + accidentReport.id  + "'", new AsyncResponder(getCarsResult, queryFault));
    }

    private function getCarsResult(qr:QueryResult):void
    {
      if (model.selectedAccidentReport.cars == null)
      {
        // store Cars
        var cars:ArrayCollection = new ArrayCollection();

        for each (var o:Object in qr.records)
        {
          cars.addItem(new Car(o));
        }

        if (cars.length != 2)
        {
          cars.removeAll();
          cars.addItem(new Car({StartX__c: 155, StartY__c: 39, EndX__c: 155, EndY__c: 122}));
          cars.addItem(new Car({StartX__c: 180, StartY__c: 314, EndX__c: 180, EndY__c: 244}));
        }

        model.selectedAccidentReport.cars = cars;
      }
    }

    private function queryFault(fault:Fault):void
    {
      model.statusMessage = "Could not load the data: " + fault.faultstring;
    }

    public function saveAccidentReport(accidentReport:AccidentReport):void
    {
      var accidentReportSObject:SObject = new SObject("AccidentReport__c");
      accidentReportSObject.Id = accidentReport.id;
      accidentReportSObject.IncidentX__c = accidentReport.incidentX;
      accidentReportSObject.IncidentY__c = accidentReport.incidentY;
      conn.update([accidentReportSObject], new AsyncResponder(saveAccidentReportResult, saveAccidentReportFault));

      var car1SObject:SObject = new SObject("IncidentCar__c");
      car1SObject.Id = accidentReport.cars[0].id;
      car1SObject.StartX__c = accidentReport.cars[0].startX;
      car1SObject.StartY__c = accidentReport.cars[0].startY;
      car1SObject.EndX__c = accidentReport.cars[0].endX;
      car1SObject.EndY__c = accidentReport.cars[0].endY;

      var car2SObject:SObject = new SObject("IncidentCar__c");
      car2SObject.Id = accidentReport.cars[1].id;
      car2SObject.StartX__c = accidentReport.cars[1].startX;
      car2SObject.StartY__c = accidentReport.cars[1].startY;
      car2SObject.EndX__c = accidentReport.cars[1].endX;
      car2SObject.EndY__c = accidentReport.cars[1].endY;

      if ((car1SObject.Id != null) && (car2SObject.Id != null))
      {
        conn.update([car1SObject, car2SObject], new AsyncResponder(saveAccidentReportResult, saveAccidentReportFault));
      }
      else
      {
        car1SObject.Id = "";
        car1SObject.AccidentReport__c = accidentReport.id;
        car2SObject.Id = "";
        car2SObject.AccidentReport__c = accidentReport.id;

        conn.create([car1SObject, car2SObject], new AsyncResponder(saveAccidentReportResult, saveAccidentReportFault));
      }

    }

    private function saveAccidentReportResult(o:Object):void
    {
      model.statusMessage = "Record Saved!";
    }

    private function saveAccidentReportFault(fault:Fault):void
    {
      model.statusMessage = "Problem Saving Record: " + fault.faultstring;
    }

  }
}

