package cars
{
  import flash.events.Event;
  import flash.events.MouseEvent;
  import flash.geom.Point;
  
  import mx.containers.Canvas;
  import mx.core.Container;
  import mx.core.UIComponent;

  public class DraggableCars extends UIComponent
  {
    public var car1:CarShape;
    public var car2:CarShape;
    public var carColor:uint;
    public var carAlpha:Number;

    private var draggingCar:CarShape;
    private var draggingCarX:Number;
    private var draggingCarY:Number;

    private var _car:Car;

    public function set car(_car:Car):void
    {
      this._car = _car;

      updateProperties();

      invalidateDisplayList();
    }

    public function get car():Car
    {
      return _car;
    }


    private function updateProperties():void
    {
      if ((car != null) && (draggingCar == null))
      {
        if (car1 != null)
        {
          car1.x = car.startX;
          car1.y = car.startY;
        }

        if (car2 != null)
        {
          car2.x = car.endX;
          car2.y = car.endY;
        }
      }
    }


    override protected function createChildren():void
    {
      super.createChildren();

      if (!car1)
      {
        car1 = new CarShape();
        car1.addEventListener(MouseEvent.MOUSE_DOWN, beginMoveCar);

        car1.carAlpha = carAlpha;
        car1.carColor = carColor;

        car1.label = "1";

        addChild(car1);
      }

      if (!car2)
      {
        car2 = new CarShape();
        car2.addEventListener(MouseEvent.MOUSE_DOWN, beginMoveCar);

        car2.carAlpha = carAlpha;
        car2.carColor = carColor;

        car2.label = "2";

        addChild(car2);
      }

      updateProperties();
    }

    override protected function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
    {
      super.updateDisplayList(unscaledWidth, unscaledHeight);

      graphics.clear();
      graphics.lineStyle(1, carColor, carAlpha, true);
      graphics.moveTo(car1.x, car1.y);

      var cX:Number = car1.x;
      var cY:Number = car2.y;

      graphics.curveTo(cX, cY, car2.x, car2.y);

      var xDist:Number = car1.x - car2.x;
      var carAngle:Number = 0;

      if (xDist < 0)
      {
        carAngle = Math.max(xDist, -90);
      }
      else
      {
        carAngle = Math.min(xDist, 90);
      }

      if (car1.y > car2.y)
      {
        carAngle = carAngle * -1;
      }

      car2.rotateCar(carAngle);
    }

    private function beginMoveCar(event:MouseEvent):void
    {
      draggingCar = event.currentTarget as CarShape;
      draggingCarX = event.localX;
      draggingCarY = event.localY;

      systemManager.addEventListener(MouseEvent.MOUSE_MOVE, moveCar);
      systemManager.addEventListener(MouseEvent.MOUSE_UP, endMoveCar);
    }

    private function moveCar(event:MouseEvent):void
    {
      var p:Point = globalToContent(new Point(event.stageX, event.stageY));
      draggingCar.move(p.x - draggingCarX, p.y - draggingCarY);

      invalidateDisplayList();
    }

    private function endMoveCar(event:MouseEvent):void
    {
      if (draggingCar == car1)
      {
        car.startX = car1.x;
        car.startY = car1.y;
      }
      else if (draggingCar == car2)
      {
        car.endX = car2.x;
        car.endY = car2.y;
      }

      draggingCar = null;

      systemManager.removeEventListener(MouseEvent.MOUSE_MOVE, moveCar);
      systemManager.removeEventListener(MouseEvent.MOUSE_UP, endMoveCar);
    }
  }
}

